/*
 * File         : spmd_Location.C
 * Author       : Guansong Zhang
 * Created      : Mon Oct 13 12:03:03 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/02/20 17:00:06 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "spmd_Location.h"

#include "adlib.h"

#include "apgStack.h"

extern jfieldID hCPPObjID;

Range * spmd_Range_ResolvePtr(JNIEnv *, jobject);
Dimension * spmd_Dimension_ResolvePtr(JNIEnv *, jobject);

#include "jctable.h"
jctable<Location> spmd_Location_Table;


/*ResolvePtr
 */
Location * spmd_Location_ResolvePtr(JNIEnv *env, jobject jthis) {
  return(spmd_Location_Table.ObjPtr[env->GetIntField(jthis, hCPPObjID)]);
}

/*
 * Class:     spmd_Location
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Location_init
  (JNIEnv * env, jclass thisClass)
{
  jclass InterfaceObject_class=env->FindClass("spmd/InterfaceObject");
  hCPPObjID=env->GetFieldID(InterfaceObject_class, "hCPPObj", "I");
  jctable<Location> *tempspmd_Location_Table=new jctable<Location>;
  spmd_Location_Table=*tempspmd_Location_Table;
  delete tempspmd_Location_Table;
};

/*
 * Class:     spmd_Location
 * Method:    clean
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Location_clean
  (JNIEnv *, jclass)
{
//  delete spmd_Location_ObjPtr;
//  delete spmd_Location_ObjRef;
};

/*
 * Class:     spmd_Location
 * Method:    constructor
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Location_constructor__
  (JNIEnv * env, jobject jthis)
{
  Location *cppLocation=new Location();
  long pos=spmd_Location_Table.NewHandle(cppLocation);

  if (spmd_Location_Table.ObjPtr[pos]==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  //AdlibPrintf("spmd_Location_AllocLength %d\n",spmd_Location_AllocLength ); // debug 
  //AdlibPrintf("new location handle is %d\n", pos); // debug

  //spmd_Location_IncRefCntr(pos);
  
  env->SetIntField(jthis, hCPPObjID, pos);
};

/*
 * Class:     spmd_Location
 * Method:    constructor
 * Signature: (Lspmd/Range;I)V
 */
JNIEXPORT void JNICALL Java_spmd_Location_constructor__Lspmd_Range_2I
  (JNIEnv *env, jobject jthis, jobject x, jint glb)
{
  Location *cppLocation=new Location(*spmd_Range_ResolvePtr(env,x),glb);
  long pos=spmd_Location_Table.NewHandle(cppLocation);

  if (spmd_Location_Table.ObjPtr[pos]==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  //spmd_Location_IncRefCntr(pos);

  env->SetIntField(jthis, hCPPObjID, pos);
}

/*
 * Class:     spmd_Location
 * Method:    ta
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Location_ta
  (JNIEnv *env, jobject jthis)
{
  apgStack.pop(&apg) ;
}

/*
 * Class:     spmd_Location
 * Method:    at
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_spmd_Location_at
  (JNIEnv * env, jobject jthis)
{
  apgStack.push(&apg);

  Location * location=spmd_Location_ResolvePtr(env, jthis);

  apg.restrict(location->dim);

  if (location->dim.crd()==location->crd)
    return (JNI_TRUE);
  else
    return (JNI_FALSE);

  return(JNI_TRUE);
};

/*
 * Class:     spmd_Location
 * Method:    dim
 * Signature: ()Lspmd/Dimension;
 */
JNIEXPORT jobject JNICALL Java_spmd_Location_dim
  (JNIEnv * env, jobject jthis)
{
  Dimension cppDim = (spmd_Location_ResolvePtr(env, jthis)->dim);

  jclass ccDimension=NULL;
  ccDimension=env->FindClass("spmd/Dimension");

  if (env->ExceptionOccurred()) return 0;

  jmethodID handleConstructorID = env->GetMethodID(ccDimension, "<init>", "()V");
  jobject jDimension=env->NewObject(ccDimension, handleConstructorID);

  Dimension * newCPPdimPtr=spmd_Dimension_ResolvePtr(env, jDimension);

  *newCPPdimPtr=cppDim;

  return(jDimension);
};

/*
 * Class:     spmd_Location
 * Method:    crd
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_spmd_Location_crd
  (JNIEnv * env, jobject jthis)
{
  return(spmd_Location_ResolvePtr(env, jthis)->crd);
};

/*
 * Class:     spmd_Location
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Location_finalize
  (JNIEnv * env, jobject jthis)
{
  long thecppobj=env->GetIntField(jthis, hCPPObjID);
  long rc=spmd_Location_Table.DecRefCntr(thecppobj);
  if (rc==0) {
    delete spmd_Location_Table.ObjPtr[thecppobj];
    spmd_Location_Table.ObjPtr[thecppobj]=NULL;
  }
};

/*
 * Class:     spmd_Location
 * Method:    show
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_spmd_Location_show
  (JNIEnv * env, jobject jthis, jstring name)
{
  int length=env->GetStringUTFLength(name);

  char * cname=new char[length+1];

  const char * jnamePtr=env->GetStringUTFChars(name, 0);

  for (int i=0; i<length; i++)
    cname[i]=*(jnamePtr+i);
  cname[length]='\0';

  env->ReleaseStringUTFChars(name, jnamePtr);

  int pos =  env->GetIntField(jthis, hCPPObjID);

  AdlibPrintf("Location %s, Handle %d, Address %x\n", cname, pos, spmd_Location_ResolvePtr(env, jthis));

  delete [] cname;

};
